@@ -113,10 +113,6 @@ gboolean            gdm_display_get_seat_id                    (GdmDisplay *disp
 gboolean            gdm_display_is_local                       (GdmDisplay *display,
                                                                 gboolean   *local,
                                                                 GError    **error);
-void                gdm_display_get_timed_login_details        (GdmDisplay *display,
-                                                                gboolean   *enabled,
-                                                                char      **username,
-                                                                int        *delay);
 gboolean            gdm_display_is_initial                     (GdmDisplay  *display,
                                                                 gboolean    *initial,
                                                                 GError     **error);
