@@ -51,6 +51,8 @@
 #include "gdm-settings-keys.h"
 
 #define INITIAL_SETUP_USERNAME "gnome-initial-setup"
+#define GDM_SESSION_MODE "gdm"
+#define INITIAL_SETUP_SESSION_MODE "initial-setup"
 #define GNOME_SESSION_SESSIONS_PATH DATADIR "/gnome-session/sessions"
 
 extern char **environ;
@@ -70,6 +72,7 @@ struct GdmLaunchEnvironmentPrivate
 
         char           *session_id;
         char           *session_type;
+        char           *session_mode;
         char           *x11_display_name;
         char           *x11_display_seat_id;
         char           *x11_display_device;
@@ -82,6 +85,7 @@ enum {
         PROP_0,
         PROP_VERIFICATION_MODE,
         PROP_SESSION_TYPE,
+        PROP_SESSION_MODE,
         PROP_X11_DISPLAY_NAME,
         PROP_X11_DISPLAY_SEAT_ID,
         PROP_X11_DISPLAY_DEVICE,
@@ -110,109 +114,6 @@ static void     gdm_launch_environment_finalize      (GObject
 
 G_DEFINE_TYPE (GdmLaunchEnvironment, gdm_launch_environment, G_TYPE_OBJECT)
 
-static void
-load_lang_config_file (const char  *config_file,
-                       const char **str_array)
-{
-        gchar         *contents = NULL;
-        gchar         *p;
-        gchar         *str_joinv;
-        gchar         *pattern;
-        gchar         *key;
-        gchar         *value;
-        gsize          length;
-        GError        *error;
-        GString       *line;
-        GRegex        *re;
-
-        g_return_if_fail (config_file != NULL);
-        g_return_if_fail (str_array != NULL);
-
-        if (!g_file_test (config_file, G_FILE_TEST_EXISTS)) {
-                g_debug ("Cannot access '%s'", config_file);
-                return;
-        }
-
-        error = NULL;
-        if (!g_file_get_contents (config_file, &contents, &length, &error)) {
-                g_debug ("Failed to parse '%s': %s",
-                         config_file,
-                         (error && error->message) ? error->message : "(null)");
-                g_error_free (error);
-                return;
-        }
-
-        if (!g_utf8_validate (contents, length, NULL)) {
-                g_warning ("Invalid UTF-8 in '%s'", config_file);
-                g_free (contents);
-                return;
-        }
-
-        str_joinv = g_strjoinv ("|", (char **) str_array);
-        if (str_joinv == NULL) {
-                g_warning ("Error in joined");
-                g_free (contents);
-                return;
-        }
-
-        pattern = g_strdup_printf ("(?P<key>(%s))=(\")?(?P<value>[^\"]*)?(\")?",
-                                   str_joinv);
-        error = NULL;
-        re = g_regex_new (pattern, 0, 0, &error);
-        g_free (pattern);
-        g_free (str_joinv);
-        if (re == NULL) {
-                g_warning ("Failed to regex: %s",
-                           (error && error->message) ? error->message : "(null)");
-                g_error_free (error);
-                g_free (contents);
-                return;
-        }
-
-        line = g_string_new ("");
-        for (p = contents; p && *p; p = g_utf8_find_next_char (p, NULL)) {
-                gunichar ch;
-                GMatchInfo *match_info = NULL;
-
-                ch = g_utf8_get_char (p);
-                if ((ch != '\n') && (ch != '\0')) {
-                        g_string_append_unichar (line, ch);
-                        continue;
-                }
-
-                if (line->str && g_utf8_get_char (line->str) == '#') {
-                        goto next_line;
-                }
-
-                if (!g_regex_match (re, line->str, 0, &match_info)) {
-                        goto next_line;
-                }
-
-                if (!g_match_info_matches (match_info)) {
-                        goto next_line;
-                }
-
-                key = g_match_info_fetch_named (match_info, "key");
-                value = g_match_info_fetch_named (match_info, "value");
-
-                if (key && *key && value && *value) {
-                        g_setenv (key, value, TRUE);
-                } else if (key && *key) {
-                        g_unsetenv (key);
-                }
-
-                g_free (key);
-                g_free (value);
-next_line:
-                g_match_info_free (match_info);
-                g_string_set_size (line, 0);
-        }
-
-        g_string_free (line, TRUE);
-        g_regex_unref (re);
-        g_free (contents);
-}
-
 static GHashTable *
 build_launch_environment (GdmLaunchEnvironment *launch_environment,
                           gboolean              start_session)
@@ -229,9 +130,6 @@ build_launch_environment (GdmLaunchEnvironment *launch_environment,
         char *system_data_dirs;
         int i;
 
-        load_lang_config_file (LANG_CONFIG_FILE,
-                               (const char **) optional_environment);
-
         /* create a hash table of current environment, then update keys has necessary */
         hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
@@ -257,6 +155,9 @@ build_launch_environment (GdmLaunchEnvironment *launch_environment,
         if (launch_environment->priv->x11_authority_file != NULL)
                 g_hash_table_insert (hash, g_strdup ("XAUTHORITY"), g_strdup (launch_environment->priv->x11_authority_file));
 
+        if (launch_environment->priv->session_mode != NULL)
+                g_hash_table_insert (hash, g_strdup ("GNOME_SHELL_SESSION_MODE"), g_strdup (launch_environment->priv->session_mode));
+
         g_hash_table_insert (hash, g_strdup ("LOGNAME"), g_strdup (launch_environment->priv->user_name));
         g_hash_table_insert (hash, g_strdup ("USER"), g_strdup (launch_environment->priv->user_name));
         g_hash_table_insert (hash, g_strdup ("USERNAME"), g_strdup (launch_environment->priv->user_name));
@@ -528,7 +429,6 @@ gdm_launch_environment_stop (GdmLaunchEnvironment *launch_environment)
         }
 
         if (launch_environment->priv->session != NULL) {
-                gdm_session_stop_conversation (launch_environment->priv->session, "gdm-launch-environment");
                 gdm_session_close (launch_environment->priv->session);
 
                 g_clear_object (&launch_environment->priv->session);
@@ -566,6 +466,14 @@ _gdm_launch_environment_set_session_type (GdmLaunchEnvironment *launch_environme
         launch_environment->priv->session_type = g_strdup (session_type);
 }
 
+static void
+_gdm_launch_environment_set_session_mode (GdmLaunchEnvironment *launch_environment,
+                                          const char           *session_mode)
+{
+        g_free (launch_environment->priv->session_mode);
+        launch_environment->priv->session_mode = g_strdup (session_mode);
+}
+
 static void
 _gdm_launch_environment_set_x11_display_name (GdmLaunchEnvironment *launch_environment,
                                               const char           *name)
@@ -654,6 +562,9 @@ gdm_launch_environment_set_property (GObject      *object,
         case PROP_SESSION_TYPE:
                 _gdm_launch_environment_set_session_type (self, g_value_get_string (value));
                 break;
+        case PROP_SESSION_MODE:
+                _gdm_launch_environment_set_session_mode (self, g_value_get_string (value));
+                break;
         case PROP_X11_DISPLAY_NAME:
                 _gdm_launch_environment_set_x11_display_name (self, g_value_get_string (value));
                 break;
@@ -704,6 +615,9 @@ gdm_launch_environment_get_property (GObject    *object,
         case PROP_SESSION_TYPE:
                 g_value_set_string (value, self->priv->session_type);
                 break;
+        case PROP_SESSION_MODE:
+                g_value_set_string (value, self->priv->session_mode);
+                break;
         case PROP_X11_DISPLAY_NAME:
                 g_value_set_string (value, self->priv->x11_display_name);
                 break;
@@ -763,6 +677,13 @@ gdm_launch_environment_class_init (GdmLaunchEnvironmentClass *klass)
                                                               NULL,
                                                               NULL,
                                                               G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_SESSION_MODE,
+                                         g_param_spec_string ("session-mode",
+                                                              NULL,
+                                                              NULL,
+                                                              NULL,
+                                                              G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
                                          PROP_X11_DISPLAY_NAME,
                                          g_param_spec_string ("x11-display-name",
@@ -928,6 +849,7 @@ create_gnome_session_environment (const char *session_id,
                                   const char *display_name,
                                   const char *seat_id,
                                   const char *session_type,
+                                  const char *session_mode,
                                   const char *display_hostname,
                                   gboolean    display_is_local)
 {
@@ -964,6 +886,7 @@ create_gnome_session_environment (const char *session_id,
                                            "command", command,
                                            "user-name", user_name,
                                            "session-type", session_type,
+                                           "session-mode", session_mode,
                                            "x11-display-name", display_name,
                                            "x11-display-seat-id", seat_id,
                                            "x11-display-hostname", display_hostname,
@@ -984,16 +907,12 @@ gdm_create_greeter_launch_environment (const char *display_name,
 {
         const char *session_name = NULL;
 
-        if (g_strcmp0 (session_type, "wayland") == 0)
-                session_name = "gnome-wayland";
-        else if (g_strcmp0 (session_type, "x11") == 0)
-                session_name = "gnome";
-
         return create_gnome_session_environment (session_name,
                                                  GDM_USERNAME,
                                                  display_name,
                                                  seat_id,
                                                  session_type,
+                                                 GDM_SESSION_MODE,
                                                  display_hostname,
                                                  display_is_local);
 }
@@ -1009,6 +928,7 @@ gdm_create_initial_setup_launch_environment (const char *display_name,
                                                  display_name,
                                                  seat_id,
                                                  NULL,
+                                                 INITIAL_SETUP_SESSION_MODE,
                                                  display_hostname,
                                                  display_is_local);
 }
